﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.SymbolicExpressionTreeEncoding {
  [Item("ChangeNodeTypeMove", "A move on a symbolic expression tree that represents an change of a single node symbol.")]
  [StorableClass]
  public class ChangeNodeTypeMove : Item {

    [Storable]
    public ISymbolicExpressionTree Tree { get; set; }

    [Storable]
    public ISymbolicExpressionTreeNode Parent { get; set; }

    [Storable]
    public int SubtreeIndex { get; set; }

    [Storable]
    public ISymbolicExpressionTreeNode NewChild { get; set; }


    [StorableConstructor]
    protected ChangeNodeTypeMove(bool deserializing) : base(deserializing) { }
    protected ChangeNodeTypeMove(ChangeNodeTypeMove original, Cloner cloner)
      : base(original, cloner) {
      this.Tree = cloner.Clone(original.Tree);
      this.SubtreeIndex = original.SubtreeIndex;
      this.Parent = cloner.Clone(original.Parent);
      this.NewChild = cloner.Clone(original.NewChild);
    }
    public ChangeNodeTypeMove(ISymbolicExpressionTree tree, ISymbolicExpressionTreeNode parent, int subtreeIndex, ISymbolicExpressionTreeNode newChild)
      : base() {
      this.Tree = tree;
      this.SubtreeIndex = subtreeIndex;
      this.Parent = parent;
      this.NewChild = newChild;
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ChangeNodeTypeMove(this, cloner);
    }
  }
}
